#include "cmnFile.h"
#include "cmnString.h"
#include "cmnException.h"

namespace utils
{
    CFile::CFile()
        : m_handle(0)
    {}
    CFile::CFile(const char * pFullName)
        : m_handle(0)
    {
        Init(std::string(pFullName));
    }
    CFile::CFile(const wchar_t * pFullName)
        : m_handle(0)
    {
        Init(std::wstring(pFullName));
    }

    CFile::CFile(const std::wstring & fullname, DWORD dwDesiredAccess, DWORD dwShareAccess, DWORD dwDisposition, DWORD dwFlagsAndAttributes)
        : m_handle(0)
    {
        InitCustom(fullname, dwDesiredAccess, dwShareAccess, dwDisposition, dwFlagsAndAttributes);
    }

    CFile::~CFile()
    {
        Close();
    }


    void CFile::Init(HANDLE handle)
    {
        Close();
        m_handle = handle;
    }

    void CFile::Init(const std::string & fullname)
    {
        Close();
        HANDLE handle = CreateFileA(fullname.c_str(), 
            GENERIC_READ, 
            FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
            0,
            OPEN_EXISTING,
            0,
            0);
        if (handle == INVALID_HANDLE_VALUE)
        {
            DWORD lastError = GetLastError();
            throw utils::CWin32Exception("Cannot open file: "+fullname, lastError);
        }
        m_handle = handle;
    }

    void CFile::Init(const std::wstring & fullname)
    {
        Close();
        HANDLE handle = CreateFileW(fullname.c_str(), 
            GENERIC_READ, 
            FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
            0,
            OPEN_EXISTING,
            0,
            0);
        if (handle == INVALID_HANDLE_VALUE)
        {
            DWORD lastError = GetLastError();
            throw utils::CWin32Exception("Cannot open file: "+utils::wstring2string(fullname), lastError);
        }
        m_handle = handle;
    }

    void CFile::InitCustom( const std::wstring & fullname,   
                            DWORD dwDesiredAccess, 
                            DWORD dwShareAccess, 
                            DWORD dwDisposition, 
                            DWORD dwFlagsAndAttributes )
    {
        Close();
        HANDLE handle = CreateFileW(fullname.c_str(), 
            dwDesiredAccess, 
            dwShareAccess,
            0,
            dwDisposition,
            dwFlagsAndAttributes,
            0);
        if (handle == INVALID_HANDLE_VALUE)
        {
            DWORD lastError = GetLastError();
            throw utils::CWin32Exception("Cannot open file: "+utils::wstring2string(fullname), lastError);
        }
        m_handle = handle;
    }

    void CFile::Write(const void * pData, DWORD dwSize)
    {
        DWORD written = 0;
        if (!WriteFile(m_handle, pData, dwSize, &written, 0))
        {
            DWORD dwErr = GetLastError(); 
            throw utils::CWin32Exception("CFile.WriteFileFail", dwErr);
        }
        if (written != dwSize)
        {
            DWORD dwErr = GetLastError(); 
            throw utils::CWin32Exception("CFile.WriteFileFailSize", dwErr);
        }
    }

    void CFile::ReadExact(void * pData, DWORD dwSize)
    {
        if (!dwSize)
            return;

        DWORD read = 0;
        if (!ReadFile(m_handle, pData, dwSize, &read, 0))
        {
            DWORD dwErr = GetLastError(); 
            throw utils::CWin32Exception("CFile.ReadExactFileFail", dwErr);
        }
        if (read != dwSize)
        {
            DWORD dwErr = GetLastError(); 
            throw utils::CWin32Exception("CFile.ReadExactFileFailSize", dwErr);
        }
    }

    DWORD CFile::Read(void * pData, DWORD dwSize)
    {
        DWORD read = 0;
        if (!ReadFile(m_handle, pData, dwSize, &read, 0))
        {
            DWORD dwErr = GetLastError();
            if ( dwErr == ERROR_HANDLE_EOF )
            {
                return 0;
            }
            throw utils::CWin32Exception("CFile.ReadFileFail", dwErr);
        }
        return read;
    }

    HANDLE CFile::getHandle()
    {
        return m_handle;
    }

    void CFile::Close()
    {
        if (m_handle)
        {
            CloseHandle( m_handle );
        }
        m_handle = 0;
    }

}