#ifndef __CMN_STRING__
#define __CMN_STRING__

#include <iostream>
#include <vector>
#include <Windows.h>
#include "cmnException.h"

namespace utils
{
    std::string wstring2string(const std::wstring &from);
    std::string wstring2string(const wchar_t* from, size_t size);
    std::wstring string2wstring(const std::string &from);
    std::wstring string2wstring(const char* from, size_t size);


    inline void wchar2str(const wchar_t * str, size_t strSize, std::vector<char> * res, UINT cpId = CP_ACP)
    {
        if(!strSize)
            return;
        res->resize(strSize + 1);//it is enough in 99%
        while(1)
        {
            int size = WideCharToMultiByte(cpId, 0, str, (int)strSize, &(*res)[0], (int)res->size(), 0, 0);
            if(size)
            {
                res->resize(size + 1);
                return;
            }
            DWORD error = ::GetLastError();
            if(error != ERROR_INSUFFICIENT_BUFFER)
                throw utils::CWin32Exception ( __FUNCTION__, error );
            size = WideCharToMultiByte(cpId, 0, str, (int)strSize, 0, 0, 0, 0);
            if(!size)
                throw utils::CWin32Exception ( __FUNCTION__, ::GetLastError() );
            res->resize(size + 1);
        }
    }

    inline void wchar2str_unthrowable(const wchar_t * str, size_t strSize, std::vector<char> * res, UINT cpId = CP_ACP)
    {
        try
        {
            wchar2str(str, strSize, res, cpId);
        }
        catch(const std::exception& ){}//for capability
    }

    //-----------------------------------------------------------------------------
    template<class wcharVectorType>
    inline void char2wstr(const char * str, size_t strSize, wcharVectorType * res, UINT cpId = CP_ACP)
    {
        if(!strSize)
            return;
        res->resize(strSize + 1);//it is enough in 99.99%
        while(1)
        {
            int size = MultiByteToWideChar(cpId, 0, str, (int)strSize, &res->front(), (int)res->size());
            if(size)
            {
                res->resize(size + 1);
                return;
            }
            DWORD error = ::GetLastError();
            if(error != ERROR_INSUFFICIENT_BUFFER)
                throw utils::CWin32Exception ( __FUNCTION__, error );
            size = MultiByteToWideChar(cpId, 0, str, (int)strSize, 0, 0);
            if(!size)
                throw utils::CWin32Exception ( __FUNCTION__, ::GetLastError() );
            res->resize(size + 1);
        }
    }

    //-----------------------------------------------------------------------------
    inline void char2wstr_unthrowable(const char * str, size_t strSize, std::vector<wchar_t> * res, UINT cpId = CP_ACP)
    {
        try
        {
            char2wstr(str, strSize, res, cpId);
        }
        catch(const std::exception& ){}//for capability
    }
}

#endif // __CMN_STRING__